Release 970525

Tue May 20 19:20:23 1997  Pablo Saratxaga <srtxg@linux.chanae.stben.be>

	* [resources/sysres_Es.rc]
	Updated CHOOSE_FONT, CHOOSE_COLOR, EDITMENU for Spanish.

Mon May 19 22:06:04 1997  Michiel van Loon <mfvl@xs4all.nl>

	* [multimedia/mcistring.c]
	Corrected bug for device!element command.

	* [multimedia/mmaux.c]
	Replaced printf and fprintf calls by dprintf_mmaux.

	* [multimedia/audio.c]
	Corrected debugmessage in wodGetVolume.
	Include code for MCI_CUE command.

	* [multimedia/mmsystem.c]
	Added the MCIERR_SEQ error messages.

	* [if1632/mmsystem.spec] [multimedia/audio.c] [multimedia/mmsystem.c]
	Changed call structure of waveInOpen and waveOutOpen.

	* [multimedia/mmsystem.c] [multimedia/audio.c] [multimedia/midi.c]
	  [multimedia/mmaux.c] [multimedia/mcicda.c] [multimedia/mcianim.c]
	  [multimedia/mcistring.c] [include/mmsystem.h]
	Changed the deviceID scheme.

	* [include/queue.h] [include/win16drv.h] [msdos/dpmi.c]
	  [windows/user.c] [windows/driver.c] [graphic/wing.c]
	  [graphics/x11drv/bitmap.c] [misc/wsprintf.c] [misc/crtdll.c]
	Removed compiler warnings.

Mon May 19 01:32:24 1997  Alex Korobka <alex@trantor.pharm.sunysb.edu>

	* [controls/menu.c] [windows/win.c] [windows/graphics.c]
	Popup menu shade, new system menu implementation, 
	ModifyMenu() fixes, better check mark painting.

	* [windows/mdi.c]
	MDI client fix for Win32.

Sat May 17 12:02:11 1997  Albrecht Kleine  <kleine@ak.sax.de>

	* [objects/metafile.c]
	Added handling of META_DIBBITBLT, META_SETTEXTJUSTIFICATION
	plus bugfix in META_EXTTEXTOUT (start_of_text etc.)

Thu May 15 22:52:00 1997  Jimen Ching  <jching@flex.com>

	* [loader/ne_image.c]
	Make sure dgroup is valid by checking pModule->flags consistently.
diff --git a/objects/dib.c b/objects/dib.c
index 0eb78f0..a951745 100644
--- a/objects/dib.c
+++ b/objects/dib.c
@@ -18,53 +18,51 @@
 
 extern void CLIPPING_UpdateGCRegion(DC* );
 
+static int	bitmapDepthTable[] = { 8, 1, 32, 16, 24, 15, 4, 0 };
+static int	ximageDepthTable[] = { 0, 0, 0,  0,  0,  0,  0 };
+
 /***********************************************************************
- *           DIB_GetImageWidthBytesX11
- *
- * Return the width of an X image in bytes
+ *           DIB_Init
  */
-int DIB_GetImageWidthBytesX11( int width, int depth )
+BOOL32 DIB_Init()
 {
-    int		wbits;
-    XImage	*testimage;
+    int		i;
+    XImage*	testimage;
 
-#define DEPTHCASE(depth) 						\
-    case depth: {							\
-	static int w##depth = 0;					\
-	if (! w##depth ) {						\
-	    testimage=XCreateImage(display,DefaultVisualOfScreen(screen),\
-		   depth,ZPixmap,0,NULL,1,1,32,20);			\
-	    w##depth = testimage->bits_per_pixel;			\
-	    XDestroyImage(testimage);					\
-	}								\
-	wbits = width*w##depth;						\
-	break;								\
-    }
-
-    switch(depth)
+    for( i = 0; bitmapDepthTable[i]; i++ )
     {
-    	DEPTHCASE(1);
-	DEPTHCASE(4);
-	DEPTHCASE(8);
-	DEPTHCASE(15);
-	DEPTHCASE(16);
-	DEPTHCASE(24);
-	DEPTHCASE(32);
-	default:
-        	fprintf(stderr, "DIB: unsupported depth %d.\n", depth );
-	/* assume 32 bits/pixel */
-	        wbits = width*32;
-		break;
+	 testimage = XCreateImage(display, DefaultVisualOfScreen(screen),
+			 bitmapDepthTable[i], ZPixmap, 0, NULL, 1, 1, 32, 20 );
+	 if( testimage ) ximageDepthTable[i] = testimage->bits_per_pixel;
+	 else return FALSE;
+	 XDestroyImage(testimage);
     }
-    return 4 * ((wbits+31)/32);
+    return TRUE;
 }
 
 /***********************************************************************
- *           DIB_GetImageWidthBytes
+ *           DIB_GetXImageWidthBytes
  *
  * Return the width of an X image in bytes
  */
-int DIB_GetImageWidthBytes( int width, int depth )
+int DIB_GetXImageWidthBytes( int width, int depth )
+{
+    int		i;
+
+    for( i = 0; bitmapDepthTable[i] ; i++ )
+	 if( bitmapDepthTable[i] == depth )
+	     return (4 * ((width * ximageDepthTable[i] + 31)/32));
+    fprintf(stderr, "DIB: unsupported depth %d.\n", depth );
+    return (4 * width);
+}
+
+/***********************************************************************
+ *           DIB_GetDIBWidthBytes
+ *
+ * Return the width of a DIB bitmap in bytes. DIB bitmap data is 32-bit aligned.
+ * http://www.microsoft.com/msdn/sdk/platforms/doc/sdk/win32/struc/src/str01.htm
+ */
+int DIB_GetDIBWidthBytes( int width, int depth )
 {
     int words;
 
@@ -90,7 +88,7 @@
 /***********************************************************************
  *           DIB_BitmapInfoSize
  *
  * Return the size of the bitmap info structure.
  */
 int DIB_BitmapInfoSize( BITMAPINFO * info, WORD coloruse )
 {
@@ -791,6 +789,8 @@
 
 /***********************************************************************
  *           GetDIBits32    (GDI32.170)
+ *
+ * http://www.microsoft.com/msdn/sdk/platforms/doc/sdk/win32/func/src/f30_14.htm
  */
 INT32 GetDIBits32( HDC32 hdc, HBITMAP32 hbitmap, UINT32 startscan,
                    UINT32 lines, LPSTR bits, BITMAPINFO * info,
@@ -864,7 +864,7 @@
 	switch( info->bmiHeader.biBitCount )
 	{
 	   case 8:
-		/* pad up to 32 bit (FIXME: not 16? ) */
+		/* pad up to 32 bit */
 		pad += (4 - (info->bmiHeader.biWidth & 3)) & 3;
 		for( y = yend - 1; (int)y >= (int)startscan; y-- )
 		{
@@ -953,8 +953,8 @@
 	info->bmiHeader.biPlanes = 1;
 	info->bmiHeader.biBitCount = bmp->bitmap.bmBitsPixel;
 	info->bmiHeader.biSizeImage = bmp->bitmap.bmHeight *
-                             DIB_GetImageWidthBytes( bmp->bitmap.bmWidth,
-                                                     bmp->bitmap.bmBitsPixel );
+                             DIB_GetDIBWidthBytes( bmp->bitmap.bmWidth,
+                                                   bmp->bitmap.bmBitsPixel );
 	info->bmiHeader.biCompression = 0;
     }
@@ -90,7 +88,7 @@
  *           DIB_BitmapInfoSize
  *
- * Return the size of the bitmap info structure.
+ * Return the size of the bitmap info structure including color table.
  */
 int DIB_BitmapInfoSize( BITMAPINFO * info, WORD coloruse )